package nianioTest;

import nianio.AppCommand;
import nianio.AppMessage;
import nianio.Event;
import nianio.ExternalCommand;
import nianio.complib.CompLibIdEvent;
import nianio.genrap.GenrapPluginEvent;
import nianioTest.complib.CompLibPlugin;
import nianioTest.genrap.GenrapPlugin;
import ocean.common.Err;

public class MsgDispatcher {
	private final CompLibPlugin compLib;
	private final GenrapPlugin genrap;
	private final AppPlugin app;

	public MsgDispatcher() {
		this.compLib = CompLibPlugin.create(CompLibIdEvent.MMConv.asIntf(new CompLibEventInvoker()));
		this.genrap = GenrapPlugin.create(new GenrapEventInvoker());
		this.app = AppPlugin.create(new ExternalCommandInvoker());
	}

	public void post(AppMessage cmd) {
		doPost(cmd);
	}

	private void doPost(AppMessage cmd) {
		if (cmd.isAppNianio()) {
			app.post(cmd.asAppNianio());
		} else if (cmd.isPlugin()) {
			ExternalCommand extCmd = cmd.asPlugin();
			if (extCmd.isComplib()) {
				compLib.post(extCmd.asComplib());
			} else if (extCmd.isGenrap()) {
				genrap.post(extCmd.asGenrap());
			} else if (extCmd.isExit()) {
				doExit();
			} else
				throw Err.errM(extCmd);
		} else
			throw Err.errM(cmd);
	}

	private void doExit() {
		genrap.dispose();
		System.exit(0);
	}

	private class CompLibEventInvoker implements CompLibIdEvent.MM {
		public void invoke(CompLibIdEvent event) {
			doPost(AppMessage.createAppNianio(AppCommand.createEvent(Event.createComplib(event))));
		}
	}

	private class GenrapEventInvoker implements GenrapPluginEvent.MM {
		public void invoke(GenrapPluginEvent event) {
			doPost(AppMessage.createAppNianio(AppCommand.createEvent(Event.createGenrap(event))));
		}
	}

	private class ExternalCommandInvoker implements ExternalCommand.MM {
		public void invoke(ExternalCommand cmd) {
			doPost(AppMessage.createPlugin(cmd));
		}
	}
}
