package nianioTest.complib;

import genrap.ui.appLogic.fmtr.FontsMetrics;
import genrap.ui.appLogic.resources.Icons32x32;
import genrap.ui.compLib.j2se.impl.AwtCompsFactory;
import genrap.ui.framework.CompHandle;
import genrap.ui.framework.FrameworkCreator;
import genrap.ui.framework.ImageSet;
import genrap.ui.framework.ParentComp;
import genrap.ui.host.j2se.fmtr.FmtrFactory;
import genrap.ui.host.resources.Images;
import nianio.complib.CompLibCommand;
import nianio.complib.CompLibIdEvent;
import nianio.complib.CompState;
import nianio.complib.Comps;
import nianio.complib.DialogCommand;
import nianio.complib.DialogIdEvent;
import nianio.complib.DialogState;
import nianio.complib.FrameCommand;
import nianio.complib.FrameState;
import nianio.complib.MessageDialogIdEvent;
import nianio.complib.PanelState;
import nianio.complib.ShownFrameIdCommand;
import nianio.complib.UserDialogCommand;
import nianio.complib.UserDialogIdCommand;
import nianio.complib.CompLibIdEvent.MMIntf;
import nianio.complib.controls.ButtonEvent;
import nianio.complib.controls.ButtonIdCommand;
import nianio.complib.controls.ControlCommand;
import nianio.complib.controls.ControlIdEvent;
import nianio.complib.controls.ControlState;
import nianio.complib.controls.LabelIdCommand;
import nianio.complib.controls.RadioButtonEvent;
import nianio.complib.controls.RadioButtonGroupEvent;
import nianio.complib.controls.RadioButtonGroupIdCommand;
import nianio.complib.controls.RadioButtonIdCommand;
import nianio.complib.controls.TextBoxEvent;
import nianio.complib.controls.TextBoxIdCommand;
import nianio.complib.filechooser.FileChooserCommand;
import nianio.complib.filechooser.FileChooserEvent;
import nianio.complib.filechooser.FileChooserIdParams;
import nianio.complib.messagedialog.MessageDialogCommand;
import nianio.complib.messagedialog.MessageDialogEvent;
import nianio.complib.messagedialog.MessageDialogIdParams;
import nianio.ids.ButtonId;
import nianio.ids.DialogId;
import nianio.ids.FileChooserId;
import nianio.ids.MessageDialogId;
import nianio.ids.RadioButtonGroupId;
import nianio.ids.RadioButtonId;
import nianio.ids.TextBoxId;
import nianio.ids.WindowId;
import nianioTest.complib.controls.Panel;
import nianioTest.complib.repository.ButtonRepository;
import nianioTest.complib.repository.DialogRepository;
import nianioTest.complib.repository.FrameRepository;
import nianioTest.complib.repository.LabelRepository;
import nianioTest.complib.repository.PanelRepository;
import nianioTest.complib.repository.RadioButtonGroupRepository;
import nianioTest.complib.repository.TextBoxRepository;
import nianioTest.complib.windows.FileChooserDialog;
import nianioTest.complib.windows.MessageDialog;
import ocean.common.Err;

public class CompLibInvoker {
	private final TextBoxRepository textBoxes;
	private final ButtonRepository buttons;
	private final PanelRepository panels;
	private final LabelRepository labels;
	private final RadioButtonGroupRepository radioButtonGroups;
	private final DialogRepository dialogs;
	private final FrameRepository frames;
	private final AwtCompsFactory cFactory;
	private final MMIntf invoker;

	public CompLibInvoker(CompLibIdEvent.MMIntf invoker) {
		ImageSet imageSet = FrameworkCreator.createImageSet(Images.getImages(),
				Images.getImages32x32(), Images.getCursors(),
				Images.getAppImages());
		FontsMetrics fm = FmtrFactory.createSystemFontsMetrics();
		this.cFactory = AwtCompsFactory.createCompsFactory(imageSet,
				Icons32x32.APP_MAIN_ICON,
				fm.getFontMS(genrap.ui.appLogic.resources.Config.FONT_SCALE));
		ControlEventInvoker controlEventInvoker = new ControlEventInvoker(
				invoker);
		DialogEventInvoker dialogInvoker = new DialogEventInvoker(invoker);
		this.textBoxes = new TextBoxRepository(controlEventInvoker);
		this.buttons = new ButtonRepository(controlEventInvoker);
		this.labels = new LabelRepository();
		this.panels = new PanelRepository();
		this.frames = new FrameRepository(invoker);
		this.radioButtonGroups = new RadioButtonGroupRepository(
				controlEventInvoker, cFactory);
		this.dialogs = new DialogRepository(dialogInvoker);
		this.invoker = invoker;
	}

	private CompHandle createComp(CompState state) {
		if (state.isPanel()) {
			return createPanel(state.asPanel());
		} else if (state.isControl()) {
			return createControl(state.asControl());
		} else
			throw Err.errM(state);
	}

	private CompHandle createPanel(PanelState state) {
		Panel p = panels.addPanel(state.getId(), state.getLayout());
		Comps comps = state.getComps();
		for (int i = 0, L = comps.size(); i < L; i++) {
			p.add(createComp(comps.get(i)));
		}
		return p.getHandle();
	}

	private CompHandle createControl(ControlState state) {
		if (state.isButton()) {
			return buttons.addButton(state.asButton());
		} else if (state.isLabel()) {
			return labels.addLabel(state.asLabel());
		} else if (state.isRadioButtonGroup()) {
			return radioButtonGroups.addRadioButtonGroup(state.asRadioButtonGroup());
		} else if (state.isTextBox()) {
			return textBoxes.addTextBox(state.asTextBox());
		} else
			throw Err.errM(state);
	}

	private void addDialog(DialogState state) {
		CompHandle content = createComp(state.getContent());
		ParentComp parent = getParent(state.getParentId());
		dialogs.addDialog(state, content, parent);
	}

	private ParentComp getParent(WindowId parentId) {
		if (parentId.isFrame()) {
			return frames.getParentComp(parentId.asFrame());
		} else if (parentId.isDialog()) {
			DialogId d = parentId.asDialog();
			if (d.isFileChooser() || d.isMessage()) {
				throw Err.err("Invalid dialog");
			} else if (d.isUser()) {
				return dialogs.getParentComp(d.asUser());
			} else
				throw Err.errM(d);
		} else
			throw Err.errM(parentId);
	}

	private void addFrame(FrameState state) {
		CompHandle content = createComp(state.getContent());
		frames.addFrame(state, content);
	}

	private void createFileChooser(FileChooserIdParams state) {
		ParentComp parentComp = getParent(state.getParentId());
		FileChooserDialog.show(state.getParams(), parentComp,
				FileChooserEvent.MMConv.asIntf(new FileChooserEventInvoker(
						state.getId())));
	}

	private void createMessageDialog(MessageDialogIdParams state) {
		ParentComp parentComp = getParent(state.getParentId());
		MessageDialog.show(state, parentComp,
				MessageDialogEvent.MMConv.asIntf(new MessageDialogEventInvoker(
						state.getId())));
	}

	private class ControlEventInvoker implements ControlIdEvent.MMIntf {
		private final CompLibIdEvent.MMIntf invoker;

		public ControlEventInvoker(CompLibIdEvent.MMIntf invoker) {
			this.invoker = invoker;
		}

		public void buttonEvent(ButtonId id, ButtonEvent cmd) {
			invoker.control(ControlIdEvent.mkButtonEvent(id, cmd));
		}

		public void radioButtonEvent(RadioButtonId id, RadioButtonEvent cmd) {
			invoker.control(ControlIdEvent.mkRadioButtonEvent(id, cmd));
		}

		public void radioButtonGroupIdEvent(RadioButtonGroupId id,
				RadioButtonGroupEvent cmd) {
			invoker.control(ControlIdEvent.mkRadioButtonGroupIdEvent(id, cmd));
		}

		public void textBoxEvent(TextBoxId id, TextBoxEvent cmd) {
			invoker.control(ControlIdEvent.mkTextBoxEvent(id, cmd));
		}
	}

	private class DialogEventInvoker implements DialogIdEvent.MM {
		private final CompLibIdEvent.MMIntf invoker;

		public DialogEventInvoker(CompLibIdEvent.MMIntf invoker) {
			this.invoker = invoker;
		}

		public void invoke(DialogIdEvent event) {
			invoker.dialog(event);
		}
	}

	private class FileChooserEventInvoker implements FileChooserEvent.MM {
		final private FileChooserId id;

		public FileChooserEventInvoker(FileChooserId id) {
			this.id = id;
		}

		public void invoke(FileChooserEvent event) {
			invoker.dialog(DialogIdEvent.mkFileChooser(id, event));
		}
	}

	private class MessageDialogEventInvoker implements MessageDialogEvent.MM {
		final private MessageDialogId id;

		public MessageDialogEventInvoker(MessageDialogId id) {
			this.id = id;
		}

		public void invoke(MessageDialogEvent event) {
			invoker.dialog(DialogIdEvent.createMessageDialog(MessageDialogIdEvent.create(
					id, event)));
		}
	}

	public void invoke(CompLibCommand cmd) {
		if (cmd.isControl()) {
			invoke(cmd.asControl());
		} else if (cmd.isDialog()) {
			invoke(cmd.asDialog());
		} else if (cmd.isFrame()) {
			invoke(cmd.asFrame());
		} else
			throw Err.errM(cmd);
	}

	private void invoke(FrameCommand command) {
		if (command.isCreateAndShow()) {
			addFrame(command.asCreateAndShow());
		} else if (command.isFrame()) {
			invoke(command.asFrame());
		} else
			throw Err.errM(command);
	}

	private void invoke(ShownFrameIdCommand command) {
		frames.get(command.getId()).post(command.getCmd());
	}

	private void invoke(DialogCommand command) {
		if (command.isFileChooser()) {
			invoke(command.asFileChooser());
		} else if (command.isMessageDialog()) {
			invoke(command.asMessageDialog());
		} else if (command.isUserDialog()) {
			invoke(command.asUserDialog());
		} else
			throw Err.errM(command);
	}

	private void invoke(UserDialogIdCommand command) {
		UserDialogCommand r = command.getCmd();
		if (r.isCreateDialog()) {
			addDialog(r.asCreateDialog());
		} else if (r.isDialog()) {
			dialogs.getDialog(command.getId()).post(r.asDialog());
		} else
			throw Err.errM(r);
	}

	private void invoke(MessageDialogCommand command) {
		if (command.isShow()) {
			createMessageDialog(command.asShow());
		} else
			throw Err.errM(command);

	}

	private void invoke(FileChooserCommand command) {
		if (command.isCreateAndShow()) {
			createFileChooser(command.asCreateAndShow());
		} else
			throw Err.errM(command);
	}

	private void invoke(ControlCommand command) {
		if (command.isTextBox()) {
			invoke(command.asTextBox());
		} else if (command.isLabel()) {
			invoke(command.asLabel());
		} else if (command.isButton()) {
			invoke(command.asButton());
		} else if (command.isRadioButton()) {
			invoke(command.asRadioButton());
		} else if (command.isRadioButtonGroup()) {
			invoke(command.asRadioButtonGroup());
		} else 		
			throw Err.errM(command);
	}

	private void invoke(TextBoxIdCommand command) {
		textBoxes.get(command.getId()).post(command.getCmd());
	}

	private void invoke(LabelIdCommand command) {
		labels.get(command.getId()).post(command.getCmd());
	}

	private void invoke(ButtonIdCommand command) {
		buttons.get(command.getId()).post(command.getCmd());
	}

	private void invoke(RadioButtonIdCommand command) {
		Err.info("There shouldn't be any radio button command");
	}

	private void invoke(RadioButtonGroupIdCommand command) {
		radioButtonGroups.get(command.getId()).post(command.getCmd());
	}

}
