package nianioTest.complib.windows;

import genrap.ui.framework.ParentComp;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import nianio.complib.filechooser.Extensions;
import nianio.complib.filechooser.FileChooserEvent;
import nianio.complib.filechooser.FileChooserKind;
import nianio.complib.filechooser.FileChooserParams;
import ocean.common.Err;

public class FileChooserDialog {
	public static void show(FileChooserParams params, ParentComp parent, FileChooserEvent.MMIntf invoker) {
		String defaultPath = params.tryGetPath();
		Extensions extensions = params.getExtensions();
		String description = params.getDescription();
		FileFilter filter = new ExtensionFileFilter(extensions, description);
		JFileChooser fileChooser = new JFileChooser(defaultPath);
		fileChooser.setFileFilter(filter);
		FileChooserKind kind = params.getKind();
		int returnValue = showFileChooser(fileChooser, kind, parent);
		if (returnValue == JFileChooser.APPROVE_OPTION) {
			invoker.fileSelected(fileChooser.getSelectedFile().getPath());
		} else
			invoker.fileSelected(null);
	}
	
	private static int showFileChooser(JFileChooser fileChooser, FileChooserKind kind,
			ParentComp parent) {
		if (kind.isOpen()) {
			return fileChooser.showOpenDialog(tryGetNativeParent(parent));
		} else if (kind.isSave()) {
			return fileChooser.showSaveDialog(tryGetNativeParent(parent));
		} else
			throw Err.errM(kind);
	}

	private static Component tryGetNativeParent(ParentComp parentComp) {
		if (parentComp.isDialog())
			return parentComp.asDialog();
		else if (parentComp.isFrame())
			return parentComp.asFrame();
		else
			return null;
	}
	public static class ExtensionFileFilter extends FileFilter {
		private final Extensions _extensions;
		private final String _description;

		public ExtensionFileFilter(Extensions extensions, String description) {
			_extensions = extensions;
			_description = description;
		}

		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			String extension = getExtension(f.getName());
			if (extension != null && _extensions.find(extension) != -1)
				return true;
			return false;
		}

		public String getDescription() {
			return _description;
		}

		public static String getExtension(String s) {
			String ext = null;
			int i = s.lastIndexOf('.');
			if (i > 0 && i < s.length() - 1) {
				ext = s.substring(i + 1).toLowerCase();
			}
			return ext;
		}
	}

}
