package nianioTest.genrap;

import java.io.File;
import java.io.IOException;

import nianio.genrap.GenrapPluginCommand;
import nianio.genrap.GenrapPluginEvent;
import nianio.genrap.ReportParams;
import ocean.common.Err;

import com.comarch.genrap.api.engine.GenRapEngine;
import com.comarch.genrap.api.engine.GenRapEngineImpl;

public class GenrapInvoker {
	public static GenrapPluginEvent invoke(GenrapPluginCommand cmd) {
		if (cmd.isGenerateReport()) {
			return generateReport(cmd.asGenerateReport());
		} else
			throw Err.errM(cmd);
	}

	private static GenrapPluginEvent generateReport(ReportParams params) {
		String reportFileName = params.getReportFileName();
		String xmlFileName = params.getXmlFileName();
		String xsdFileName = params.getXsdFileName();
		String outputFileName = params.getOutputFileName();

		if (!canReadFile(reportFileName)) {
			String msg = "Cannot read report file: \""
					+ params.getReportFileName() + "\"";
			return GenrapPluginEvent.mkFailure(params, msg);
		}
		if (!canReadFile(xmlFileName)) {
			String msg = "Cannot read xml file: \"" + params.getXmlFileName()
					+ "\"";
			return GenrapPluginEvent.mkFailure(params, msg);
		}
		if (!canReadFile(xsdFileName)) {
			String msg = "Cannot read xsd file: \"" + params.getXsdFileName()
					+ "\"";
			return GenrapPluginEvent.mkFailure(params, msg);
		}
		if (!canWriteFile(outputFileName)) {
			String msg = "Cannot write to file: \""
					+ params.getOutputFileName() + "\"";
			return GenrapPluginEvent.mkFailure(params, msg);
		}
		GenRapEngine engine = new GenRapEngineImpl();
		engine.setLanguage("en");
		engine.setReportFilePath(reportFileName);
		engine.setExportOutFilePath(outputFileName);
		engine.setDataSource("xml::uri=(." + xmlFileName + ");xsd=(."
				+ xsdFileName + ");");
		try {
			if (params.getOutputKind().isPdf()) {
				engine.exportToPDF();
			} else if (params.getOutputKind().isHtml()) {
				engine.exportToHTML();
			} else if (params.getOutputKind().isXls()) {
				engine.exportToXLS();
			} else
				throw Err.errM(params);
		} catch (Exception e) {
			return GenrapPluginEvent.mkFailure(params, e.getMessage());
		}
		return GenrapPluginEvent.createSuccess(params);
	}

	private static boolean canReadFile(String path) {
		File file = new File(path);
		return file.canRead();
	}

	private static boolean canWriteFile(String path) {
		File file = new File(path);
		if (file.exists())
			return file.canWrite();
		try {
			if (file.createNewFile())
				return file.delete();
			return false;
		} catch (IOException e) {
			return false;
		}
	}
}
