package nianioTest.genrap;

import nianio.genrap.GenrapPluginCommand;
import nianio.genrap.GenrapPluginCommands;
import nianio.genrap.GenrapPluginEvent;
import ocean.common.Err;
import ocean.common.Monitor;

public class GenrapPlugin {
	private final GenrapPluginEvent.MM deliverer;
	private final Monitor lock = new Monitor();
	private final Thread thread;

	private GenrapPluginCommands queue = GenrapPluginCommands.EMPTY;
	private boolean close = false;

	private GenrapPlugin(GenrapPluginEvent.MM deliverer) {
		this.deliverer = deliverer;
		this.thread = new Thread(new Runnable() {
			public void run() {
				doWork();
			}
		}, "GenrapPlugin");
		thread.start();
	}

	private void doWork() {
		for (;;) {
			GenrapPluginCommand cmd;
			synchronized (lock) {
				if (queue.size() == 0)
					lock.waitFully();
				if (queue.size() == 0 || close) {
					System.out.println("Genrap plugin finished");
					return;
				}
				cmd = queue.get(0);
				queue = queue.remove(0);
			}
			doInvoke(cmd);
		}
	}

	private void doInvoke(GenrapPluginCommand cmd) {
		try {
			GenrapPluginEvent res = GenrapInvoker.invoke(cmd);
			deliverer.invoke(res);
		} catch (Throwable e) {
			Err.info(e);
		}
	}

	public static GenrapPlugin create(GenrapPluginEvent.MM deliverer) {
		return new GenrapPlugin(deliverer);
	}

	public void post(GenrapPluginCommand cmd) {
		synchronized (lock) {
			boolean weak = queue.size() == 0;
			queue = queue.add(cmd);
			if (weak)
				lock.notify();
		}
	}
	
	public void dispose(){
		synchronized (lock) {
			close = true;
			lock.notifyAll();
		};
	}

}
