package nianioTest.nianio.impl;

import nianio.AppState;
import nianio.Entry;
import nianio.MsgDialogs;
import nianio.SelectFileCmd;
import nianio.SelectFileExtCmds;
import nianio.Status;
import nianio.AppLogicExtCmd.MMIntf;
import nianio.complib.filechooser.Extensions;
import nianio.complib.filechooser.FileChooserKind;
import nianio.complib.messagedialog.MessageDialogKind;
import nianio.complib.messagedialog.MessageDialogParams;
import nianio.genrap.GenrapPluginEvent;
import nianio.genrap.OutputKind;
import nianio.genrap.ReportParams;
import nianioTest.nianio.helper.MessageHelper;
import nianioTest.nianio.types.AppLogicCommandInvokerIntf;
import nianioTest.nianio.types.SelectFileExtCmdAcc;
import ocean.common.Err;

public class AppLogicCommandInvoker implements AppLogicCommandInvokerIntf {

	public AppState run(AppState state, MMIntf deliverer) {
		deliverer.showMainFrame(state.getXmlFile().getFileName(),
				state.getXsdFile().getFileName(), state.getReportFile()
						.getFileName(), state.getOutputFile().getFileName(),
				state.getOutputKind(),
				MessageHelper.getStatusString(state.getStatus()));
		return state;
	}

	public AppState close(AppState state, MMIntf deliverer) {
		deliverer.closeMainFrame();
		deliverer.exit();
		return state;
	}

	public AppState setPdfOutput(AppState state, MMIntf deliverer) {
		return setOutputKind(state, OutputKind.createPdf());
	}

	private AppState setOutputKind(AppState state, OutputKind kind) {
		return state.setOutputKind(kind);
	}

	public AppState setHtmlOutput(AppState state, MMIntf deliverer) {
		return setOutputKind(state, OutputKind.createHtml());
	}

	public AppState setXlsOutput(AppState state, MMIntf deliverer) {
		return setOutputKind(state, OutputKind.createXls());
	}

	public AppState generateRequest(AppState state, MMIntf deliverer) {
		deliverer.generate(ReportParams.create(
				state.getXmlFile().getFileName(), state.getXsdFile()
						.getFileName(), state.getReportFile().getFileName(),
				state.getOutputFile().getFileName(), state.getOutputKind()));
		return state.setStatus(incStatus(state.getStatus()));
	}

	private Status incStatus(Status status) {
		if (status.isGeneratingReport()) {
			return Status.createGeneratingReport(status.asGeneratingReport() + 1);
		} else if (status.isReady()) {
			return Status.createGeneratingReport(1);
		} else
			throw Err.errM(status);
	}

	private Status decStatus(Status status) {
		if (status.isGeneratingReport()) {
			int newStat = status.asGeneratingReport() - 1;
			if (newStat <= 0)
				return Status.createReady();
			return Status.createGeneratingReport(newStat);
		} else if (status.isReady()) {
			return status;
		} else
			throw Err.errM(status);
	}

	public AppState reportGenerated(AppState state, GenrapPluginEvent value,
			MMIntf deliverer) {
		MessageDialogParams params;
		if (value.isSuccess()) {
			params = MessageDialogParams.create(MessageDialogKind.createInfo(),
					MessageHelper.getSuccessMsg(value.asSuccess()));
		} else if (value.isFailure()) {
			params = MessageDialogParams.create(MessageDialogKind.createInfo(),
					MessageHelper.getFailureMsg(value.asFailure()));
		} else
			throw Err.errM(value);
		MsgDialogs messageDialogStates = state.getMessageDialogStates();
		String key = getKey(messageDialogStates.size());
		state = state.setMessageDialogStates(messageDialogStates.put(createEntry(
				params, key)));
		state = state.setStatus(decStatus(state.getStatus()));
		deliverer.showMessage(key, params);
		return state;
	}

	private static Entry createEntry(MessageDialogParams params, String key) {
		return Entry.create(key, params);
	}

	private static String getKey(int sid) {
		return "" + sid;
	}

	public AppState reportFileCmd(AppState state, SelectFileCmd value,
			MMIntf deliverer) {
		SelectFileExtCmdAcc acc = new SelectFileExtCmdAcc();
		state = state.setReportFile(SelectFileNianioImpl.create(
				FileChooserKind.createOpen(), Extensions.EMPTY.add("grs"),
				"Report files").invoke(state.getReportFile(), value,
				acc.getMMIntf()));
		SelectFileExtCmds cmds = acc.getResult();
		for (int i = 0; i < cmds.size(); i++) {
			deliverer.reportFile(cmds.get(i));
		}
		return state;
	}

	public AppState xmlFileCmd(AppState state, SelectFileCmd value,
			MMIntf deliverer) {
		SelectFileExtCmdAcc acc = new SelectFileExtCmdAcc();
		state = state.setXmlFile(SelectFileNianioImpl.create(
				FileChooserKind.createOpen(), Extensions.EMPTY.add("xml"),
				"XML files").invoke(state.getXmlFile(), value, acc.getMMIntf()));
		SelectFileExtCmds cmds = acc.getResult();
		for (int i = 0; i < cmds.size(); i++) {
			deliverer.xmlFile(cmds.get(i));
		}
		return state;
	}

	public AppState xsdFileCmd(AppState state, SelectFileCmd value,
			MMIntf deliverer) {
		SelectFileExtCmdAcc acc = new SelectFileExtCmdAcc();
		state = state.setXsdFile(SelectFileNianioImpl.create(
				FileChooserKind.createOpen(), Extensions.EMPTY.add("xsd"),
				"XSD files").invoke(state.getXsdFile(), value, acc.getMMIntf()));
		SelectFileExtCmds cmds = acc.getResult();
		for (int i = 0; i < cmds.size(); i++) {
			deliverer.xsdFile(cmds.get(i));
		}
		return state;
	}

	public AppState outputFileCmd(AppState state, SelectFileCmd value,
			MMIntf deliverer) {
		SelectFileExtCmdAcc acc = new SelectFileExtCmdAcc();
		state = state.setOutputFile(SelectFileNianioImpl.create(
				FileChooserKind.createSave(), getOutputFileExtensions(state),
				getOutputFileDescription(state)).invoke(state.getOutputFile(),
				value, acc.getMMIntf()));
		SelectFileExtCmds cmds = acc.getResult();
		for (int i = 0; i < cmds.size(); i++) {
			deliverer.outputFile(cmds.get(i));
		}
		return state;
	}

	private static Extensions getOutputFileExtensions(AppState state) {
		OutputKind outputKind = state.getOutputKind();
		Extensions exts = Extensions.EMPTY;
		if (outputKind.isHtml()) {
			return exts.add("htm").add("html");
		} else if (outputKind.isPdf()) {
			return exts.add("pdf");
		} else if (outputKind.isXls()) {
			return exts.add("xls");
		} else
			throw Err.errM(outputKind);
	}

	private static String getOutputFileDescription(AppState state) {
		OutputKind outputKind = state.getOutputKind();
		if (outputKind.isHtml()) {
			return "HTML files";
		} else if (outputKind.isPdf()) {
			return "PDF files";
		} else if (outputKind.isXls()) {
			return "XLS files";
		} else
			throw Err.errM(outputKind);
	}

	public AppState messageDialogClosed(AppState state, MMIntf deliverer,
			String key) {
		return state.setMessageDialogStates(state.getMessageDialogStates()
				.remove(key));
	}
	/*	{
	 
	 if (cmd.isSetPdfOutput()) {
	 return setOutputKind(state, OutputKind.createPdf(), deliverer);
	 } else if (cmd.isSetHtmlOutput()) {
	 return setOutputKind(state, OutputKind.createHtml(), deliverer);
	 } else if (cmd.isSetXlsOutput()) {
	 return setOutputKind(state, OutputKind.createXls(), deliverer);
	 } else if (cmd.isGenerateReport()) {
	 return generateReport(state, deliverer);
	 } else if (cmd.isReportGenerated()) {
	 return reportGenerated(state, deliverer);
	 } else if (cmd.isReportGenerationFailed()) {
	 return reportGenerationFailed(state,
	 cmd.asReportGenerationFailed(), deliverer);
	 } else if (cmd.isSetReportFileName()) {
	 return setReportFileName(state, cmd.asSetReportFileName(),
	 deliverer);
	 } else if (cmd.isSetXmlFileName()) {
	 return setDataFileName(state, cmd.asSetXmlFileName(), deliverer);
	 } else if (cmd.isSetXsdFileName()) {
	 return setSchemaFileName(state, cmd.asSetXsdFileName(), deliverer);
	 } else if (cmd.isSetOutputFileName()) {
	 return setOutputFileName(state, cmd.asSetOutputFileName(),
	 deliverer);
	 } else if (cmd.isMessageDialogClosed()) {
	 return state.setMessageDialogState(null);
	 } else if (cmd.isSelectReportFile()) {
	 Extensions exts = Extensions.EMPTY;
	 exts = exts.add("grs");
	 FileChooserParams params = FileChooserParams.create(
	 FileChooserKind.createOpen(),
	 id(ApplicationId.createOpenReportFileDialog()),
	 id(ApplicationId.createMainFrame()), null, exts,
	 "Report files");
	 showFileChooser(params, deliverer);
	 return state.setFileChooserState(params);
	 } else if (cmd.isSelectXmlFile()) {
	 Extensions exts = Extensions.EMPTY;
	 exts = exts.add("xml");
	 FileChooserParams params = FileChooserParams.create(
	 FileChooserKind.createOpen(),
	 id(ApplicationId.createOpenDataFileDialog()),
	 id(ApplicationId.createMainFrame()), null, exts,
	 "XML files");
	 showFileChooser(params, deliverer);
	 return state.setFileChooserState(params);
	 } else if (cmd.isSelectXsdFile()) {
	 Extensions exts = Extensions.EMPTY;
	 exts = exts.add("xsd");
	 FileChooserParams params = FileChooserParams.create(
	 FileChooserKind.createOpen(),
	 id(ApplicationId.createOpenSchemaFileDialog()),
	 id(ApplicationId.createMainFrame()), null, exts,
	 "XSD files");
	 showFileChooser(params, deliverer);
	 return state.setFileChooserState(params);
	 } else if (cmd.isSelectOutputFile()) {
	 Extensions extensions = getOutputFileExtensions(state);
	 String description = getOutputFileDescription(state);
	 FileChooserParams params = FileChooserParams.create(
	 FileChooserKind.createSave(),
	 id(ApplicationId.createSaveOutputFileDialog()),
	 id(ApplicationId.createMainFrame()), null, extensions,
	 description);
	 showFileChooser(params, deliverer);
	 return state.setFileChooserState(params);
	 } else if (cmd.isReportFileSelected()) {
	 String fileName = cmd.asReportFileSelected().tryGetPath();
	 if (fileName != null)
	 state = setReportFileName(state, fileName, deliverer);
	 return state.setFileChooserState(null);
	 } else if (cmd.isXmlFileSelected()) {
	 String fileName = cmd.asXmlFileSelected().tryGetPath();
	 if (fileName != null)
	 state = setDataFileName(state, fileName, deliverer);
	 return state.setFileChooserState(null);
	 } else if (cmd.isXsdFileSelected()) {
	 String fileName = cmd.asXsdFileSelected().tryGetPath();
	 if (fileName != null)
	 state = setSchemaFileName(state, fileName, deliverer);
	 return state.setFileChooserState(null);
	 } else if (cmd.isOutputFileSelected()) {
	 String fileName = cmd.asOutputFileSelected().tryGetPath();
	 if (fileName != null) {
	 if (FileChooserDialog.ExtensionFileFilter.getExtension(fileName) == null)
	 fileName += "." + getOutputFileExtension(state);
	 state = setOutputFileName(state, fileName, deliverer);
	 }
	 return state.setFileChooserState(null);
	 } else if (cmd.isRun()) {
	 return run(state, deliverer);
	 } else if (cmd.isClose()) {
	 return close(state, deliverer);
	 } else
	 throw Err.errM(cmd);
	 }

	 private static AppState setOutputKind(AppState state, OutputKind output,
	 AppLogicExtCmd.MMIntf cmds) {
	 RadioButtonGroupCommand rbgCmd = getRBGCommand(output);
	 cmds.complib(CompLibCommand.createControl(ControlCommand.createRadioButtonGroup(rbgCmd)));
	 return state.setOutputKind(output);
	 }

	 private static RadioButtonGroupCommand getRBGCommand(OutputKind output) {
	 if (output.isHtml()) {
	 return RadioButtonGroupCommand.createSelect(RadioButtonGroupSelect.create(
	 id(ApplicationId.createOutputType()),
	 id(ApplicationId.createHtmlOutput())));
	 } else if (output.isPdf()) {
	 return RadioButtonGroupCommand.createSelect(RadioButtonGroupSelect.create(
	 id(ApplicationId.createOutputType()),
	 id(ApplicationId.createPdfOutput())));
	 } else if (output.isXls()) {
	 return RadioButtonGroupCommand.createSelect(RadioButtonGroupSelect.create(
	 id(ApplicationId.createOutputType()),
	 id(ApplicationId.createXlsOutput())));
	 } else
	 throw Err.errM(output);
	 }

	 private static AppState generateReport(AppState state,
	 AppLogicExtCmd.MMIntf cmds) {
	 Id id = Id.createApplication(ApplicationId.createReportGeneration());
	 ReportParams params = ReportParams.create(state.getXmlFileName(),
	 state.getXsdFileName(), state.getReportFileName(),
	 state.getOutputFileName(), state.getOutputKind());
	 cmds.genrap(GenrapPluginCommand.create(id,
	 GenrapCommand.createGenerateReport(params)));
	 int reportsGenerating = 0;
	 Status status = state.getStatus();
	 if (status.isGeneratingReport()) {
	 reportsGenerating = status.asGeneratingReport() + 1;
	 } else if (status.isReady()) {
	 reportsGenerating = 1;
	 } else
	 throw Err.errM(status);
	 return setStatus(state,
	 Status.createGeneratingReport(reportsGenerating), cmds);
	 }

	 private static AppState reportGenerated(AppState state,
	 AppLogicExtCmd.MMIntf cmds) {
	 Status status = state.getStatus();
	 if (status.isGeneratingReport()) {
	 int reportsGenerating = status.asGeneratingReport();
	 if (reportsGenerating == 1)
	 return setStatus(state, Status.createReady(), cmds);
	 return setStatus(state,
	 Status.createGeneratingReport(reportsGenerating - 1), cmds);
	 } else if (status.isReady()) {
	 return state;
	 } else
	 throw Err.errM(status);
	 }

	 private static AppState reportGenerationFailed(AppState state,
	 String cause, AppLogicExtCmd.MMIntf cmds) {
	 Status status = state.getStatus();
	 if (status.isGeneratingReport()) {
	 int reportsGenerating = status.asGeneratingReport();
	 MessageDialogParams params = MessageDialogParams.create(
	 id(ApplicationId.createMessageDialog()),
	 id(ApplicationId.createMainFrame()),
	 MessageDialogKind.createError(), cause);
	 AppState newState = showMessageDialog(state, params, cmds);
	 if (reportsGenerating == 1)
	 return setStatus(newState, Status.createReady(), cmds);
	 return setStatus(newState,
	 Status.createGeneratingReport(reportsGenerating - 1), cmds);
	 } else if (status.isReady()) {
	 return state;
	 } else
	 throw Err.errM(status);
	 }

	 private static AppState setReportFileName(AppState state, String fileName,
	 AppLogicExtCmd.MMIntf cmds) {
	 setText(id(ApplicationId.createReportFileTextBox()), fileName, cmds);
	 return state.setReportFileName(fileName);
	 }

	 private static AppState setDataFileName(AppState state, String fileName,
	 AppLogicExtCmd.MMIntf cmds) {
	 setText(id(ApplicationId.createDataFileTextBox()), fileName, cmds);
	 return state.setXmlFileName(fileName);
	 }

	 private static AppState setSchemaFileName(AppState state, String fileName,
	 AppLogicExtCmd.MMIntf cmds) {
	 setText(id(ApplicationId.createSchemaFileTextBox()), fileName, cmds);
	 return state.setXsdFileName(fileName);
	 }

	 private static AppState setOutputFileName(AppState state, String fileName,
	 AppLogicExtCmd.MMIntf cmds) {
	 setText(id(ApplicationId.createOutputFileTextBox()), fileName, cmds);
	 return state.setOutputFileName(fileName);
	 }

	 private static void setText(Id id, String text, AppLogicExtCmd.MMIntf cmds) {
	 TextBoxCommand tbCmd = TextBoxCommand.createSetText(TextBoxSetText.create(
	 id, text));
	 cmds.complib(CompLibCommand.createControl(ControlCommand.createTextBox(tbCmd)));
	 }

	 private static void showFileChooser(FileChooserParams params,
	 AppLogicExtCmd.MMIntf cmds) {
	 cmds.complib(CompLibCommand.createCreateAndShowFileChooser(params));
	 }

	 private static AppState showMessageDialog(AppState state,
	 MessageDialogParams params, AppLogicExtCmd.MMIntf cmds) {
	 cmds.complib(CompLibCommand.createCreateAndShowMessageDialog(params));
	 return state.setMessageDialogState(params);
	 }

	 private static AppState setStatus(AppState state, Status status,
	 AppLogicExtCmd.MMIntf cmds) {
	 String text = statusString(status);
	 LabelCommand lCmd = LabelCommand.createSetText(LabelSetText.create(
	 id(ApplicationId.createStatusLabel()), text));
	 cmds.complib(CompLibCommand.createControl(ControlCommand.createLabel(lCmd)));
	 return state.setStatus(status);
	 }

	 private static CompState comp(ControlState state) {
	 return CompState.createControl(state);
	 }

	 private static Id selectedOutputId(AppState state) {
	 OutputKind kind = state.getOutputKind();
	 if (kind.isHtml()) {
	 return id(ApplicationId.createHtmlOutput());
	 } else if (kind.isPdf()) {
	 return id(ApplicationId.createPdfOutput());
	 } else if (kind.isXls()) {
	 return id(ApplicationId.createXlsOutput());
	 } else
	 throw Err.errM(kind);
	 }

	 private static String statusString(Status status) {
	 if (status.isGeneratingReport()) {
	 int reports = status.asGeneratingReport();
	 if (reports == 1)
	 return "Generating report...";
	 return "Generating " + reports + " reports...";
	 } else if (status.isReady()) {
	 return "";
	 } else
	 throw Err.errM(status);
	 }

	 public static AppState createState() {
	 return AppState.create("", "", "", "", OutputKind.createPdf(),
	 Status.createReady(), null, null);
	 }


	 private static Id id(ApplicationId appId) {
	 return Id.createApplication(appId);
	 }
	 */

}
