package nianio.complib;

import nianio.complib.controls.ControlIdEvent;
import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;
import nianio.ids.FrameId;


/** 
 * <pre>Autogenerated class CompLibIdEvent = variant {
 *	    control of ControlIdEvent  
 *	 |  dialog of DialogIdEvent  
 *	 |  frame of FrameIdEvent  
 * }</pre>
 */ 
public class CompLibIdEvent implements ImmObject {
	private static final int TYPE_HASH = -789674871;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private CompLibIdEvent(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static CompLibIdEvent createControl(ControlIdEvent value) {
		return new CompLibIdEvent(0, value);
	}

	public static CompLibIdEvent createDialog(DialogIdEvent value) {
		return new CompLibIdEvent(1, value);
	}

	public static CompLibIdEvent createFrame(FrameIdEvent value) {
		return new CompLibIdEvent(2, value);
	}


	public static CompLibIdEvent mkFrame(
			FrameId id
			, FrameEvent event
	) {
		return createFrame( FrameIdEvent.create(
							id
							, event
							 ));
	}
	

	/** @return true, when this is control */
	public boolean isControl() {
		return variant == 0;
	}

	/** @return variant value, when this is control; throws Exception otherwise */
	public ControlIdEvent asControl() {
		SchemaException.ensure(isControl());
		return (ControlIdEvent)value;
	}

	/** @return true, when this is dialog */
	public boolean isDialog() {
		return variant == 1;
	}

	/** @return variant value, when this is dialog; throws Exception otherwise */
	public DialogIdEvent asDialog() {
		SchemaException.ensure(isDialog());
		return (DialogIdEvent)value;
	}

	/** @return true, when this is frame */
	public boolean isFrame() {
		return variant == 2;
	}

	/** @return variant value, when this is frame; throws Exception otherwise */
	public FrameIdEvent asFrame() {
		SchemaException.ensure(isFrame());
		return (FrameIdEvent)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isControl()) {
			;
		} else if (isDialog()) {
			;
		} else if (isFrame()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isControl()) {
			asControl();
		} else if (isDialog()) {
			asDialog();
		} else if (isFrame()) {
			asFrame();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isControl())
			return "control" + "(" + asControl() + ")";
		if (isDialog())
			return "dialog" + "(" + asDialog() + ")";
		if (isFrame())
			return "frame" + "(" + asFrame() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(CompLibIdEvent obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof CompLibIdEvent) 
			&& this.eq((CompLibIdEvent)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	

	// --------------- Model Manipulator Infrastructure ----------------- //

	/**
	 *  Autogenerated Model Manipulator for CompLibIdEvent
	 */
	public static interface MM {
		public void invoke(CompLibIdEvent cmd);
	}
	/**
	 *  Autogenerated Model Manipulator Interface for CompLibIdEvent
	 */
	public static interface MMIntf {
		public void control(ControlIdEvent value);
		public void dialog(DialogIdEvent value);
		public void frame(
				 FrameId id
				, FrameEvent event
		);
 	}
	/**
	 *  Autogenerated Model Manipulator Converter for CompLibIdEvent
	 */
	public static class MMConv {
		public static MMIntf asIntf(MM mm) {
			return new MyMMIntf(mm);
		}
	
		public static MM asSimple(MMIntf mm) {
			return new MyMM(mm);
		}
	
		private static class MyMM implements MM {
	
			private final MMIntf mm;
	
			public MyMM(MMIntf mm) {
				this.mm = mm;
			}
	
			public void invoke(CompLibIdEvent cmd) {
				 if (cmd.isControl()) {
					mm.control(cmd.asControl());
				} else if (cmd.isDialog()) {
					mm.dialog(cmd.asDialog());
				} else if (cmd.isFrame()) {
					FrameIdEvent val = cmd.asFrame();
					mm.frame(
							 val.getId()
							, val.getEvent()
					);
				 } else
					throw ocean.common.Err.err();				
			}
		}

		private static class MyMMIntf implements MMIntf {
			private final MM mm;
	
			public MyMMIntf(MM mm) {
				this.mm = mm;
			}
		
			public void control(ControlIdEvent value) {
				mm.invoke(CompLibIdEvent.createControl(value));
			}
		
			public void dialog(DialogIdEvent value) {
				mm.invoke(CompLibIdEvent.createDialog(value));
			}
		
			public void frame(
							 FrameId id
							, FrameEvent event
			) {
				mm.invoke( CompLibIdEvent.createFrame(
								FrameIdEvent.create( 
									id, event)));		
			}
		
		}
	}
}