package nianio.complib;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import nianio.complib.controls.ControlState;
import nianio.ids.PanelId;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class CompState = variant {
 *	    panel of PanelState  
 *	 |  control of ControlState  
 * }</pre>
 */ 
public class CompState implements ImmObject {
	private static final int TYPE_HASH = 578377378;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private CompState(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static CompState createPanel(PanelState value) {
		return new CompState(0, value);
	}

	public static CompState createControl(ControlState value) {
		return new CompState(1, value);
	}


	public static CompState mkPanel(
			PanelId id
			, Layout layout
			, Comps comps
	) {
		return createPanel( PanelState.create(
							id
							, layout
							, comps
							 ));
	}
	

	/** @return true, when this is panel */
	public boolean isPanel() {
		return variant == 0;
	}

	/** @return variant value, when this is panel; throws Exception otherwise */
	public PanelState asPanel() {
		SchemaException.ensure(isPanel());
		return (PanelState)value;
	}

	/** @return true, when this is control */
	public boolean isControl() {
		return variant == 1;
	}

	/** @return variant value, when this is control; throws Exception otherwise */
	public ControlState asControl() {
		SchemaException.ensure(isControl());
		return (ControlState)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isPanel()) {
			;
		} else if (isControl()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isPanel()) {
			asPanel();
		} else if (isControl()) {
			asControl();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isPanel())
			return "panel" + "(" + asPanel() + ")";
		if (isControl())
			return "control" + "(" + asControl() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(CompState obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof CompState) 
			&& this.eq((CompState)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}