package nianio.ids;

import schema.runtime.datatypes.ImmObject;
import schema.runtime.err.SchemaException;
import ocean.common.Err;


/** 
 * <pre>Autogenerated class WindowId = variant {
 *	    frame of FrameId  
 *	 |  dialog of DialogId  
 * }</pre>
 */ 
public class WindowId implements ImmObject {
	private static final int TYPE_HASH = -1050735285;
	private int hash = 0;

	private final int variant;
	private final Object value;  


	private WindowId(int variant, Object value) {
		SchemaException.ensure(value != null);
		this.variant = variant;
		this.value = value;
	}


	public static WindowId createFrame(FrameId value) {
		return new WindowId(0, value);
	}

	public static WindowId createDialog(DialogId value) {
		return new WindowId(1, value);
	}



	/** @return true, when this is frame */
	public boolean isFrame() {
		return variant == 0;
	}

	/** @return variant value, when this is frame; throws Exception otherwise */
	public FrameId asFrame() {
		SchemaException.ensure(isFrame());
		return (FrameId)value;
	}

	/** @return true, when this is dialog */
	public boolean isDialog() {
		return variant == 1;
	}

	/** @return variant value, when this is dialog; throws Exception otherwise */
	public DialogId asDialog() {
		SchemaException.ensure(isDialog());
		return (DialogId)value;
	}



	/** The only purpose of this method is to inline its content - match pattern. */ 
	public void match() {
		if (isFrame()) {
			;
		} else if (isDialog()) {
			;
		} else 		
			throw Err.errM(this);
	}

	/** The only purpose of this method is to inline its content - 
	 * match pattern with value unpacking. 
	 */ 
	public void matchUnpack() {
		if (isFrame()) {
			asFrame();
		} else if (isDialog()) {
			asDialog();
		} else 		
			throw Err.errM(this);
	}	

	public String toString() {
		if (isFrame())
			return "frame" + "(" + asFrame() + ")";
		if (isDialog())
			return "dialog" + "(" + asDialog() + ")";
		throw SchemaException.err();
	}
	
	public boolean eq(WindowId obj) {
		if (obj == null)
			return false;
		if (this == obj)
			return true;
		return variant == obj.variant 
			&& (value == null ? (obj.value == null) 
			: (obj.value != null && value.equals(obj.value)));
	}

	public boolean equals(Object obj) {
		return (obj instanceof WindowId) 
			&& this.eq((WindowId)obj);
	}

	public int hashCode(){
		if (hash == 0) {
			int h = variant*(variant - 79) + 1601;
			if (value != null)
				h ^= value.hashCode();
			else
				h ^= TYPE_HASH;
			hash = h == 0 ? TYPE_HASH : h;
		}
		return hash;
	}

 	
	
}